from distutils.core import setup, Extension
from config import tao_dir, petsc_dir, BOPT, PETSC_ARCH
import os
del os.link

modname = 'pytao'
filename = '%s.cpp' % modname
link_args = ['-Wall', '-fomit-frame-pointer']
compile_args = link_args + ['-DPETSC_USE_BOPT_O']
build_type = BOPT+'/'+PETSC_ARCH
include_dirs = [petsc_dir+'/include', petsc_dir+'/include/mpiuni', petsc_dir+'/bmake/'+PETSC_ARCH, tao_dir, tao_dir+'/include']
library_dirs = [petsc_dir+'/lib/lib'+build_type, tao_dir+'/lib/lib'+build_type]
runtime_library_dirs = [petsc_dir+'/lib/lib'+build_type, tao_dir+'/lib/lib'+build_type]
libraries = ['taopetsc', 'tao', 'petscsnes', 'petscksp', 'petscdm', 'petscmat', 'petscvec', 'petsc', 'blas-3', 'lapack-3', 'mpiuni']
ext = Extension(modname,
                sources=[filename],
                extra_link_args=link_args,
                extra_compile_args=compile_args,
                include_dirs=include_dirs,
                library_dirs=library_dirs,
                runtime_library_dirs=runtime_library_dirs,
                libraries=libraries);

setup(author='Jason D. M. Rennie',
      author_email='jrennie@csail.mit.edu',
      description='Python interface to the Toolkit for Advanced Optimization (TAO)',
      keywords='L-BFGS,TAO,optimization,nonlinear programming,gradient descent,conjugate gradient,unconstrained minimization',
      license='Common Public License Version 1.0',
      version='0.2.3',
      url='http://people.csail.mit.edu/~jrennie/python/',
      name=modname,
      ext_modules=[ext],
      #classifiers = ['Development Status :: 3 - Alpha',
      #               'Intended Audience :: Science/Research',
      #               'License :: OSI Approved :: Common Public License',
      #               'Language :: C++',
      #               'Topic :: Scientific/Engineering'
      #               'Topic :: Scientific/Engineering :: Artificial Intelligence']
      )
