#!/usr/bin/env python

import unittest
import pytao
import cPickle
import operator
import math

infinity=1e20
zero=1e-38

def logit(x):
    try: return 1.0/(1.0+math.exp(-x))
    except OverflowError:
        if x > 0: return 1.0
        else: return 0.0            

def sum(l):
    return reduce(operator.add,l)

def sign(x):
    if x >= 0: return 1.0
    else: return -1.0

class PyTaoTest(unittest.TestCase):

    def testRosenbrock(self):
        #print 'testRosenbrock'
        alpha=99.0
        def ogfun(x):
            fobj = 0.0
            g = [0.0]*len(x)
            for i in xrange(len(x)/2):
                t1 = x[2*i+1]-x[2*i]*x[2*i]
                t2 = 1-x[2*i]
                fobj += alpha*t1*t1 + t2*t2
                g[2*i] = -4*alpha*t1*x[2*i]-2.0*t2
                g[2*i+1] = 2*alpha*t1
            return (fobj,g)
        n = 10
        v = [0.0]*n
        x = pytao.optimize(x=v,ogfun=ogfun,verbosity=0)
        self.assert_(len(x) == n)
        for i in range(n):
            self.assert_(x[i] < 1.001 and x[i] > 0.999)

    def testSimple(self):
        #print 'testSimple'
        def pytao_train(h,n):
            def ogfun(theta):
                v = theta[0]
                phi = logit(v)
                def ffun(hi,ni):
                    try:
                        return hi*math.log(phi)+(ni-hi)*math.log(1.0-phi)
                    except OverflowError:
                        return -infinity
                f = -sum(map(ffun,h,n))
                def gfun(hi,ni):
                    return hi*(1-phi)-(ni-hi)*phi
                g = [-sum(map(gfun,h,n))]
                #print 'ogfun(%s) -> (%s,%s)' % (theta,f,g)
                return (f,g)
            v = pytao.optimize(x=[0.0],ogfun=ogfun,verbosity=0,
                               fatol=1e-6,frtol=1e-6)
            return logit(v[0])
        h = cPickle.load(open('test/h.p'))
        n = cPickle.load(open('test/n.p'))
        for k in h.keys():
            xopt = float(sum(h[k]))/sum(n)
            x = pytao_train(h[k],n)
            self.assert_(x > xopt-0.00001 and x < xopt+0.00001)

    def testArgs(self):
        ret = pytao.getArgs(x=[0.0],ogfun=logit,verbosity=1,
                            fatol=1e-6,frtol=1e-6,maxits=1234);
        self.assert_(ret[5] == 1e-6)
        self.assert_(ret[6] == 1e-6)
        self.assert_(ret[9] == 1)
        self.assert_(ret[10] == 1234)

if __name__=='__main__':
    unittest.main()
